//------------------------------------------------------------------
// Create by Robert F. Nee.  Use freely.
//
// Fin drawing modules
//
//------------------------------------------------------------------

//------------------------------------------------------------------
// Draw an untapered fin of given dimensions

module fin (thickness, root, tip, sweep, width, base)
{
	// Draw the fin in the x/y, translate it so it's thickness is centered and rotate it upright
	rotate ([90, 0, 0]) translate ([0, 0, -thickness/2])
		linear_extrude (height = thickness)
			polygon (	[
				[0, 0],
				[0, root],
				[width, root - sweep],
				[width, root - sweep - tip],
				[base, 0]
			] );
}

//------------------------------------------------------------------
// Compute the length of the root edge given the other parameters for a dual taper

function dual_tapered_root (root_thickness, lead_thickness, tip_thickness, tip, width) =
	(root_thickness - lead_thickness)*tip/(tip_thickness - lead_thickness);

//------------------------------------------------------------------
// Compute the inset to make fins properly mate with curved body surface

function fin_inset (diameter, root_thickness) =
	diameter/2 - sqrt (diameter*diameter/4 - root_thickness*root_thickness/4);

//------------------------------------------------------------------
// Draw a tapered leading edge suitable for mating with a tapered fin

module tapered_leading_edge (root_thickness, lead_thickness, tip_thickness, tip, sweep, width)
{
	// Compute the taper at the tp
	tip_taper = (root_thickness - tip_thickness)/2.0;
	lead_taper = (root_thickness - lead_thickness)/2.0;

	root = dual_tapered_root (root_thickness, lead_thickness, tip_thickness, tip, width);

	fin_points = [
		[0, 0, 0], // P0
		[0, root, lead_taper], // P1
		[width, -sweep + tip, lead_taper], // P2
		[width, -sweep, tip_taper], // P3
		[0, 0, root_thickness], // P4
		[0, root, root_thickness - lead_taper], // P5
		[width, -sweep + tip, root_thickness - lead_taper], // P6
		[width, -sweep, root_thickness - tip_taper] // P7
	];

	fin_faces = [
		[0, 1, 2, 3],		// Front
		[7, 6, 5, 4],		// Back
		[1, 0, 4, 5],		// Root
		[0, 3, 7, 4],		// Bottom
		[2, 6, 7, 3],		// Top
		[5, 6, 2, 1]		// Tip
	];

	rotate ([90, 0, 0]) translate ([0, 0, -root_thickness/2])
		polyhedron (fin_points, fin_faces);
}

//------------------------------------------------------------------
// Draw a fin with a taper from root to tip

module tapered_tip_fin (root_thickness, tip_thickness, root, tip, sweep, width, base)
{
	// Compute the taper at the tp
	tip_taper = (root_thickness - tip_thickness)/2.0;

	if (base == 0) echo ("Error base must be non-zero in order to be manifold");

	// Compute the thickness at the tip of the base width
	bevel_angle = atan ((root_thickness - tip_thickness)/(2 * width));
	base_taper = base * tan (bevel_angle);

	fin_points = [
		[0, 0, 0], // P0
		[0, root, 0], // P1
		[width, root - sweep, tip_taper], // P2
		[width, root - sweep - tip, tip_taper], //P3
		[base, 0, base_taper], // P4
		[0, 0, root_thickness], // P5
		[0, root, root_thickness], // P6
		[width, root - sweep, tip_taper + tip_thickness], // P7
		[width, root - sweep - tip, tip_taper + tip_thickness], // P8
		[base, 0, root_thickness - base_taper] // P9
	];

	fin_faces = [
		[0, 1, 2, 3, 4], // Front
		[9, 8, 7, 6, 5], // Back
		[1, 0, 5, 6],		// Root
		[5, 0, 4, 9],		// Bottom
		[1, 6, 7, 2],		// Top
		[7, 8, 3, 2],		// Tip
		[4, 3, 8, 9],		// Base to Tip 
	];

	rotate ([90, 0, 0]) translate ([0, 0, -root_thickness/2])
		polyhedron (fin_points, fin_faces);
}

//------------------------------------------------------------------
// Draw a fin unit with a shoulder for mating with a body that includes a compound bevel or taper
// one from root to tip and a tapered leading edge.

module dual_taper_fin_unit (
	diameter,
	width,
	shoulder_height,
	shoulder_diameter,
	shoulder_core,
	wall_thickness,
	base_root,
	base_sweep,
	base_tip,
	lead_tip,
	root_thickness,
	lead_thickness,
	tip_thickness
)	
{
	lead_root = dual_tapered_root (root_thickness, lead_thickness, tip_thickness, lead_tip, width);

	// Body and core
	difference () {
		// Draw body in three parts, leading, main and trailing
		cylinder (
			h = base_root + lead_root, 
			d = diameter
		);

		// Core
		translate ([0, 0, -slip])
			cylinder (h = base_root + lead_root + 2 * slip, d = diameter - 2*wall_thickness);
	}

	inset = fin_inset (diameter, root_thickness);

	// Fins
	for (i = [0:3]) {
		rotate ([0, 0, i * 90]) {
			// Draw the base portion of the fin
			translate ([diameter/2 - inset, 0, 0])
				tapered_tip_fin (root_thickness, tip_thickness,
					base_root, base_tip, base_sweep, width, 0.100);

			translate ([diameter/2 - inset, 0, base_root])
				tapered_leading_edge (root_thickness, lead_thickness, tip_thickness, lead_tip, base_sweep, width);
		}
	}

	// Shoulder
	color ("blue")
	translate ([0, 0, base_root + lead_root])
		difference () {
			cylinder (
				h = shoulder_height, 
				d = shoulder_diameter
			);
			translate ([0, 0, -slip])
			cylinder (
				h = shoulder_height + 2 * slip, 
				d = shoulder_core
			);
		}
}
